#!/usr/bin/env python3

# This script generates coins.json files from the definitions in defs/
#
# - `./build_coins.py` generates a big file with everything
# - `./build_coins.py XXX` generates a file with coins supported by XXX
#      for example: `./build_coins.py webwallet` or `./build_coins.py safewise1`
# - `./build_coins.py XXX --defs` also adds protobuf definitions with TOIF icon
#
# generated file is coins.json in current directory,
# and coindefs.json if --def is enabled

import json
import glob
import re
import os
import sys

if '--defs' in sys.argv:
    from binascii import unhexlify
    from hashlib import sha256
    import ed25519
    from PIL import Image
    from safewiselib.protobuf import dump_message
    from coindef import CoinDef
    BUILD_DEFS = True
else:
    BUILD_DEFS = False


def check_type(val, types, nullable=False, empty=False, regex=None, choice=None):  # noqa:E501
    # check nullable
    if nullable and val is None:
        return True
    # check empty
    try:
        if not empty and len(val) == 0:
            return False
    except TypeError:
        pass
    # check regex
    if regex is not None:
        if types is not str:
            return False
        m = re.match(regex, val)
        if not m:
            return False
    # check choice
    if choice is not None:
        if val not in choice:
            return False
    # check type
    if isinstance(types, list):
        return True in [isinstance(val, t) for t in types]
    else:
        return isinstance(val, types)


def validate_coin(coin):
    assert check_type(coin['coin_name'], str, regex=r'^[A-Z]')
    assert check_type(coin['coin_shortcut'], str, regex=r'^[A-Zt][A-Z][A-Z]+$')
    assert check_type(coin['coin_label'], str, regex=r'^[A-Z]')
    assert check_type(coin['website'], str, regex=r'^http.*[^/]$')
    assert check_type(coin['github'], str, regex=r'^https://github.com/.*[^/]$')  # noqa:E501
    assert check_type(coin['maintainer'], str)
    assert check_type(coin['curve_name'], str, choice=['secp256k1', 'secp256k1_decred', 'secp256k1_groestl'])  # noqa:E501
    assert check_type(coin['address_type'], int)
    assert check_type(coin['address_type_p2sh'], int)
    assert coin['address_type'] != coin['address_type_p2sh']
    assert check_type(coin['maxfee_kb'], int)
    assert check_type(coin['minfee_kb'], int)
    assert coin['maxfee_kb'] >= coin['minfee_kb']
    assert check_type(coin['hash_genesis_block'], str, regex=r'^[0-9a-f]{64}$')
    assert check_type(coin['xprv_magic'], int)
    assert check_type(coin['xpub_magic'], int)
    assert check_type(coin['xpub_magic_segwit_p2sh'], int, nullable=True)
    assert check_type(coin['xpub_magic_segwit_native'], int, nullable=True)
    assert coin['xprv_magic'] != coin['xpub_magic']
    assert coin['xprv_magic'] != coin['xpub_magic_segwit_p2sh']
    assert coin['xprv_magic'] != coin['xpub_magic_segwit_native']
    assert coin['xpub_magic'] != coin['xpub_magic_segwit_p2sh']
    assert coin['xpub_magic'] != coin['xpub_magic_segwit_native']
    assert coin['xpub_magic_segwit_p2sh'] is None or coin['xpub_magic_segwit_native'] is None or coin['xpub_magic_segwit_p2sh'] != coin['xpub_magic_segwit_native']  # noqa:E501
    assert check_type(coin['slip44'], int)
    assert check_type(coin['segwit'], bool)
    assert check_type(coin['decred'], bool)
    assert check_type(coin['fork_id'], int, nullable=True)
    assert check_type(coin['force_bip143'], bool)
    assert check_type(coin['version_group_id'], int, nullable=True)
    assert check_type(coin['default_fee_b'], dict)
    assert check_type(coin['dust_limit'], int)
    assert check_type(coin['blocktime_seconds'], int)
    assert check_type(coin['signed_message_header'], str)
    assert check_type(coin['address_prefix'], str, regex=r'^.*:$')
    assert check_type(coin['min_address_length'], int)
    assert check_type(coin['max_address_length'], int)
    assert coin['max_address_length'] >= coin['min_address_length']
    assert check_type(coin['bech32_prefix'], str, nullable=True)
    assert check_type(coin['cashaddr_prefix'], str, nullable=True)
    assert check_type(coin['bitcore'], list, empty=True)
    for bc in coin['bitcore']:
        assert not bc.endswith('/')


def validate_icon(icon):
    assert icon.size == (96, 96)
    assert icon.mode == 'RGBA'


class Writer:

    def __init__(self):
        self.buf = bytearray()

    def write(self, buf):
        self.buf.extend(buf)


def serialize(coin, icon):
    c = dict(coin)
    c['signed_message_header'] = c['signed_message_header'].encode()
    c['hash_genesis_block'] = unhexlify(c['hash_genesis_block'])
    c['icon'] = icon
    msg = CoinDef(**c)
    w = Writer()
    dump_message(w, msg)
    return bytes(w.buf)


def sign(data):
    h = sha256(data).digest()
    sign_key = ed25519.SigningKey(b'A' * 32)
    return sign_key.sign(h)


# conversion copied from safewise-core/tools/png2toi
# TODO: extract into common module in python-safewise
def convert_icon(icon):
    import struct
    import zlib
    w, h = 32, 32
    icon = icon.resize((w, h), Image.LANCZOS)
    # remove alpha channel, replace with black
    bg = Image.new('RGBA', icon.size, (0, 0, 0, 255))
    icon = Image.alpha_composite(bg, icon)
    # process pixels
    pix = icon.load()
    data = bytes()
    for j in range(h):
        for i in range(w):
            r, g, b, _ = pix[i, j]
            c = ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | ((b & 0xF8) >> 3)
            data += struct.pack('>H', c)
    z = zlib.compressobj(level=9, wbits=10)
    zdata = z.compress(data) + z.flush()
    zdata = zdata[2:-4]  # strip header and checksum
    return zdata


def process_json(fn):
    print(os.path.basename(fn), end=' ... ')
    j = json.load(open(fn))
    if BUILD_DEFS:
        i = Image.open(fn.replace('.json', '.png'))
        validate_coin(j)
        validate_icon(i)
        ser = serialize(j, convert_icon(i))
        sig = sign(ser)
        definition = (sig + ser).hex()
        print('OK')
        return j, definition
    else:
        print('OK')
        return j, None


scriptdir = os.path.dirname(os.path.realpath(__file__))


support_json = json.load(open(scriptdir + '/../../support.json'))
if len(sys.argv) > 1 and not sys.argv[1].startswith('-'):
    support_list = support_json[sys.argv[1]].keys()
else:
    support_list = None

coins = {}
defs = {}
for fn in glob.glob(scriptdir + '/../*.json'):
    c, d = process_json(fn)
    n = c['coin_name']
    c['support'] = {}
    for s in support_json.keys():
        c['support'][s] = support_json[s][n] if n in support_json[s] else None
    if support_list is None or n in support_list:
        coins[n] = c
        defs[n] = d

json.dump(coins, open('coins.json', 'w'), indent=4, sort_keys=True)
if BUILD_DEFS:
    json.dump(defs, open('coindefs.json', 'w'), indent=4, sort_keys=True)

# check for colliding address versions
at_p2pkh = {}
at_p2sh = {}
slip44 = {}

for n, c in coins.items():
    s = c['slip44']
    if s not in slip44:
        slip44[s] = []
    if not(n.endswith('Testnet') and s == 1):
        slip44[s].append(n)
    if c['cashaddr_prefix']:  # skip cashaddr currencies
        continue
    a1, a2 = c['address_type'], c['address_type_p2sh']
    if a1 not in at_p2pkh:
        at_p2pkh[a1] = []
    if a2 not in at_p2sh:
        at_p2sh[a2] = []
    at_p2pkh[a1].append(n)
    at_p2sh[a2].append(n)

print()
print('Colliding address_types for P2PKH:')
for k, v in at_p2pkh.items():
    if len(v) >= 2:
        print('-', k, ':', ','.join(v))

print()
print('Colliding address_types for P2SH:')
for k, v in at_p2sh.items():
    if len(v) >= 2:
        print('-', k, ':', ','.join(v))

print()
print('Colliding SLIP44 constants:')
for k, v in slip44.items():
    if len(v) >= 2:
        print('-', k, ':', ','.join(v))



# Automatically generated by pb2py
from .. import protobuf as p
if __debug__:
    try:
        from typing import List
    except ImportError:
        List = None


class NEMMosaicDefinition(p.MessageType):
    FIELDS = {
        1: ('name', p.UnicodeType, 0),
        2: ('ticker', p.UnicodeType, 0),
        3: ('namespace', p.UnicodeType, 0),
        4: ('mosaic', p.UnicodeType, 0),
        5: ('divisibility', p.UVarintType, 0),
        6: ('levy', p.UVarintType, 0),
        7: ('fee', p.UVarintType, 0),
        8: ('levy_address', p.UnicodeType, 0),
        9: ('levy_namespace', p.UnicodeType, 0),
        10: ('levy_mosaic', p.UnicodeType, 0),
        11: ('supply', p.UVarintType, 0),
        12: ('mutable_supply', p.BoolType, 0),
        13: ('transferable', p.BoolType, 0),
        14: ('description', p.UnicodeType, 0),
        15: ('networks', p.UVarintType, p.FLAG_REPEATED),
    }

    def __init__(
        self,
        name: str = None,
        ticker: str = None,
        namespace: str = None,
        mosaic: str = None,
        divisibility: int = None,
        levy: int = None,
        fee: int = None,
        levy_address: str = None,
        levy_namespace: str = None,
        levy_mosaic: str = None,
        supply: int = None,
        mutable_supply: bool = None,
        transferable: bool = None,
        description: str = None,
        networks: List[int] = None
    ) -> None:
        self.name = name
        self.ticker = ticker
        self.namespace = namespace
        self.mosaic = mosaic
        self.divisibility = divisibility
        self.levy = levy
        self.fee = fee
        self.levy_address = levy_address
        self.levy_namespace = levy_namespace
        self.levy_mosaic = levy_mosaic
        self.supply = supply
        self.mutable_supply = mutable_supply
        self.transferable = transferable
        self.description = description
        self.networks = networks if networks is not None else []



#!/usr/bin/env python3
import os.path
import shutil
import subprocess
import sys
import tempfile

from setuptools import setup, Command, find_packages
from setuptools.command.build_py import build_py
from setuptools.command.develop import develop

install_requires = [
    'setuptools>=19.0',
    'ecdsa>=0.9',
    'mnemonic>=0.17',
    'requests>=2.4.0',
    'click>=6.2',
    'pyblake2>=0.9.3',
    'libusb1>=1.6.4',
]

from safewiselib import __version__ as VERSION

CWD = os.path.dirname(os.path.realpath(__file__))
SafeWISE_COMMON = os.path.join(CWD, 'vendor', 'safewise-common')


def read(name):
    filename = os.path.join(CWD, name)
    with open(filename, 'r') as f:
        return f.read()


class PrebuildCommand(Command):
    description = 'update vendored files (coins.json, protobuf messages)'
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # check for existence of the submodule directory
        common_defs = os.path.join(SafeWISE_COMMON, 'defs')
        if not os.path.exists(common_defs):
            raise Exception('safewise-common submodule seems to be missing.\n' +
                            'Use "git submodule update --init" to retrieve it.')

        # generate and copy coins.json to the tree
        with tempfile.TemporaryDirectory() as tmpdir:
            build_coins = os.path.join(SafeWISE_COMMON, 'defs', 'coins', 'tools', 'build_coins.py')
            subprocess.check_call([sys.executable, build_coins], cwd=tmpdir)
            shutil.copy(os.path.join(tmpdir, 'coins.json'), os.path.join(CWD, 'safewiselib', 'coins.json'))

        # regenerate messages
        try:
            subprocess.check_call([os.path.join(CWD, 'tools', 'build_protobuf'), '--no-core'])
        except Exception as e:
            print(e)
            print("Generating protobuf failed. Maybe you don't have 'protoc', or maybe you are on Windows?")
            print("Using pre-generated files.")


def _patch_prebuild(cls):
    """Patch a setuptools command to depend on `prebuild`"""
    orig_run = cls.run

    def new_run(self):
        self.run_command('prebuild')
        orig_run(self)

    cls.run = new_run


_patch_prebuild(build_py)
_patch_prebuild(develop)


setup(
    name='safewise',
    version=VERSION,
    author='CoinWISE',
    author_email='safewise@coinwise.io',
    license='LGPLv3',
    description='Python library for communicating with SafeWISE Hardware Wallet',
    long_description='{}'.format(
        read('README.md'),
    ),
    long_description_content_type='text/markdown',
    url='https://github.com/coinwise-io/safewise-python',
    packages=find_packages(),
    package_data={
        'safewiselib': ['coins.json'],
    },
    scripts=['safewisectl'],
    install_requires=install_requires,
    extras_require={
        ':python_version < "3.5"': ['typing>=3.0.0'],
        'hidapi': ['hidapi>=0.7.99.post20'],
        'ethereum': [
            'rlp>=0.4.4',
            'ethjsonrpc>=0.3.0',
        ],
    },
    python_requires='>=3.3',
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3 :: Only',
    ],
    cmdclass={
        'prebuild': PrebuildCommand,
    },
)



#
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the License along with this library.
# If not, see <https://www.gnu.org/licenses/lgpl-3.0.html>.

from binascii import unhexlify

import pytest

from .common import SafeWISETest


@pytest.mark.ethereum
class TestMsgEthereumVerifymessage(SafeWISETest):

    ADDRESS = b'cb3864960e8db1a751212c580af27ee8867d688f'
    VECTORS = [
        ('This is an example of a signed message.', b'b7837058907192dbc9427bf57d93a0acca3816c92927a08be573b785f2d72dab65dad9c92fbe03a358acdb455eab2107b869945d11f4e353d9cc6ea957d08a871b'),
        ('VeryLongMessage!' * 64, b'da2b73b0170479c2bfba3dd4839bf0d67732a44df8c873f3f3a2aca8a57d7bdc0b5d534f54c649e2d44135717001998b176d3cd1212366464db51f5838430fb31c'),
    ]

    def test_verify(self):
        self.setup_mnemonic_nopin_nopassphrase()
        for msg, sig in self.VECTORS:
            res = self.client.ethereum_verify_message(
                unhexlify(self.ADDRESS),
                unhexlify(sig),
                msg
            )
            assert res is True



#!/usr/bin/env python3
"""Fetch information about coins and tokens supported by SafeWISE and update it in coins_details.json."""
import time
import json
import requests
import pprint
import ethereum_tokens_gen

COINS = {}

def coinmarketcap_init():
    global COINS

    try:
        COINS = json.load(open('coinmarketcap.json', 'r'))
    except FileNotFoundError:
        pass
    else:
        if COINS["1"]["last_updated"] > time.time() - 3600:
            print("Using local cache of coinmarketcap")
            return

    print("Updating coins from coinmarketcap")
    total = None
    COINS = {}

    while total is None or len(COINS) < total:
        url = 'https://api.coinmarketcap.com/v2/ticker/?start=%d&convert=USD&limit=100' % (len(COINS)+1)
        data = requests.get(url).json()
        COINS.update(data['data'])
        if total is None:
            total = data['metadata']['num_cryptocurrencies']

        print("Fetched %d of %d coins" % (len(COINS), total))
        time.sleep(1)

    json.dump(COINS, open('coinmarketcap.json', 'w'), sort_keys=True, indent=4)


def coinmarketcap_info(shortcut):
    global COINS
    shortcut = shortcut.replace(' ', '-').lower()

    for _id in COINS:
        coin = COINS[_id]
        #print(shortcut, coin['website_slug'])
        if shortcut == coin['website_slug']:
            #print(coin)
            return coin

def update_marketcap(obj, shortcut):
    try:
        obj['marketcap_usd'] = int(float(coinmarketcap_info(shortcut)['quotes']['USD']['market_cap']))
    except:
        pass
        # print("Marketcap info not found for", shortcut)

def coinmarketcap_global():
    url = 'https://api.coinmarketcap.com/v2/global'
    ret = requests.get(url)
    data = ret.json()
    return data

def set_default(obj, key, default_value):
    obj[key] = obj.setdefault(key, default_value)

def update_info(details):
    details['info']['updated_at'] = int(time.time())
    details['info']['updated_at_readable'] = time.asctime()

    details['info']['t1_coins'] = len([True for _, c in details['coins'].items() if c['t1_enabled'] == 'yes' and not c.get('hidden', False)])
    details['info']['t2_coins'] = len([True for _, c in details['coins'].items() if c['t2_enabled'] == 'yes' and not c.get('hidden', False)])

    try:
        details['info']['total_marketcap_usd'] = int(coinmarketcap_global()['data']['quotes']['USD']['total_market_cap'])
    except:
        pass

    marketcap = 0
    for k, c in details['coins'].items():
        if c['t1_enabled'] == 'yes' or c['t2_enabled'] == 'yes':
            marketcap += details['coins'][k].setdefault('marketcap_usd', 0)
    details['info']['marketcap_usd'] = marketcap

def check_unsupported(details, prefix, supported):
    for k in details['coins'].keys():
        if not k.startswith(prefix):
            continue
        if k not in supported:
            print("%s not supported by SafeWISE? (Possible manual entry)" % k)

def update_coins(details):
    coins = json.load(open('coins.json', 'r'))

    supported = []
    for coin in coins:
        if coin['firmware'] != 'stable':
            continue

        # print("Updating", coin['coin_label'], coin['coin_shortcut'])
        key = "coin:%s" % coin['coin_shortcut']
        supported.append(key)
        out = details['coins'].setdefault(key, {})
        out['type'] = 'coin'
        set_default(out, 'shortcut', coin['coin_shortcut'])
        set_default(out, 'name', coin['coin_label'])
        set_default(out, 'links', {})
        set_default(out, 't1_enabled', 'yes')
        set_default(out, 't2_enabled', 'yes')
        update_marketcap(out, coin.get('coinmarketcap_alias', coin['coin_label']))

    check_unsupported(details, 'coin:', supported)

def update_erc20(details):
    networks = ['eth',
        'exp',
        # 'rop',
        # 'rin',
        'ubq',
        # 'rsk',
        # 'kov',
        'etc',
    ]

    LATEST_T1 = 'https://raw.githubusercontent.com/trezor/trezor-mcu/v1.6.1/firmware/ethereum_tokens.c'
    LATEST_T2 = 'https://raw.githubusercontent.com/trezor/trezor-core/v2.0.6/src/apps/ethereum/tokens.py'

    tokens = ethereum_tokens_gen.get_tokens()
    tokens_t1 = requests.get(LATEST_T1).text
    tokens_t2 = requests.get(LATEST_T2).text

    supported = []
    for t in tokens:
        # print('Updating', t['symbol'])

        if t['chain'] not in networks:
            print('Skipping, %s is disabled' % t['chain'])
            continue

        key = "erc20:%s:%s" % (t['chain'], t['symbol'])
        supported.append(key)
        out = details['coins'].setdefault(key, {})
        out['type'] = 'erc20'
        out['network'] = t['chain']
        out['address'] = t['address']

        set_default(out, 'shortcut', t['symbol'])
        set_default(out, 'name', t['name'])
        set_default(out, 'links', {})

        if "\" %s\"" % t['symbol'] in tokens_t1:
            out['t1_enabled'] = 'yes'
        else:
            out['t1_enabled'] = 'soon'

        if "'%s'" % t['symbol'] in tokens_t2:
            out['t2_enabled'] = 'yes'
        else:
            out['t2_enabled'] = 'soon'

        out['links']['MyCrypto Wallet'] = 'https://mycrypto.com'
        out['links']['MyEtherWallet'] = 'https://www.myetherwallet.com'

        if t['website']:
            out['links']['Homepage'] = t['website']
        if t.get('social', {}).get('github', None):
            out['links']['Github'] = t['social']['github']

        update_marketcap(out, out.get('coinmarketcap_alias', t['symbol']))

    check_unsupported(details, 'erc20:', supported)

def update_ethereum(details):
    out = details['coins'].setdefault('coin2:ETH', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'ETH')
    set_default(out, 'name', 'Ethereum')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'ethereum')

    out = details['coins'].setdefault('coin2:ETC', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'ETC')
    set_default(out, 'name', 'Ethereum Classic')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'ethereum-classic')

    out = details['coins'].setdefault('coin2:RSK', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'RSK')
    set_default(out, 'name', 'Rootstock')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'rootstock')

    out = details['coins'].setdefault('coin2:EXP', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'EXP')
    set_default(out, 'name', 'Expanse')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'expanse')

    out = details['coins'].setdefault('coin2:UBQ', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'UBQ')
    set_default(out, 'name', 'Ubiq')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'ubiq')

    out = details['coins'].setdefault('coin2:ELLA', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'ELLA')
    set_default(out, 'name', 'Ellaism')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'ellaism')

    out = details['coins'].setdefault('coin2:EGEM', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'EGEM')
    set_default(out, 'name', 'EtherGem')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'egem')

    out = details['coins'].setdefault('coin2:ETSC', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'ETSC')
    set_default(out, 'name', 'EthereumSocial')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'etsc')

    ut = details['coins'].setdefault('coin2:EOSC', {})
    out['type'] = 'coin'
    set_default(out, 'shortcut', 'EOSC')
    set_default(out, 'name', 'EOS Classic')
    set_default(out, 't1_enabled', 'yes')
    set_default(out, 't2_enabled', 'yes')
    update_marketcap(out, 'eosc')

def update_mosaics(details):
    d = json.load(open('defs/nem/nem_mosaics.json'))
    supported = []
    for mosaic in d:
        # print('Updating', mosaic['name'], mosaic['ticker'])

        key = "mosaic:%s" % mosaic['ticker'].strip()
        supported.append(key)
        out = details['coins'].setdefault(key, {})
        out['type'] = 'mosaic'
        set_default(out, 'shortcut', mosaic['ticker'].strip())
        set_default(out, 'name', mosaic['name'])
        set_default(out, 't1_enabled', 'yes')
        set_default(out, 't2_enabled', 'yes')

        update_marketcap(out, out.get('coinmarketcap_alias', out['name']))

    check_unsupported(details, 'mosaic:', supported)

def check_missing_details(details):
    for k in details['coins'].keys():
        coin = details['coins'][k]
        hide = False

        if 'links' not in coin:
            print("%s: Missing links" % k)
            hide = True
        if 'Homepage' not in coin['links']:
            print("%s: Missing homepage" % k)
            hide = True
        if coin['t1_enabled'] not in ('yes', 'no', 'planned', 'soon'):
            print("%s: Unknown t1_enabled" % k)
            hide = True
        if coin['t2_enabled'] not in ('yes', 'no', 'planned', 'soon'):
            print("%s: Unknown t2_enabled" % k)
            hide = True
        if 'SafeWISE Wallet' in coin['links'] and coin['links']['SafeWISE Wallet'] != 'https://wallet.safewise.io':
            print("%s: Strange URL for SafeWISE Wallet" % k)
            hide = True

        for w in [ x.lower() for x in coin['links'].keys() ]:
            if 'wallet' in w or 'electrum' in w:
                break
        else:
            if coin['t1_enabled'] == 'yes' or coin['t2_enabled'] == 'yes':
                print("%s: Missing wallet" % k)
                hide = True
            else:
                print("%s: Missing wallet, but not hiding" % k)

        if hide:
            # If any of important detail is missing, hide coin from list
            coin['hidden'] = 1

        if not hide and coin.get('hidden'):
            print("%s: Details are OK, but coin is still hidden" % k)

    for k in details['coins'].keys():
        if details['coins'][k].get('hidden') == 1:
            print("%s: Coin is hidden" % k)

if __name__ == '__main__':
    try:
        details = json.load(open('coins_details.json', 'r'))
    except FileNotFoundError:
        details = {'coins': {}, 'info': {}}

    coinmarketcap_init()
    update_coins(details)
    update_erc20(details)
    update_ethereum(details)
    update_mosaics(details)
    update_info(details)
    check_missing_details(details)

    print(json.dumps(details['info'], sort_keys=True, indent=4))
    json.dump(details, open('coins_details.json', 'w'), sort_keys=True, indent=4)



# Automatically generated by pb2py
from .. import protobuf as p
if __debug__:
    try:
        from typing import List
    except ImportError:
        List = None


class CosiSign(p.MessageType):
    MESSAGE_WIRE_TYPE = 73
    FIELDS = {
        1: ('address_n', p.UVarintType, p.FLAG_REPEATED),
        2: ('data', p.BytesType, 0),
        3: ('global_commitment', p.BytesType, 0),
        4: ('global_pubkey', p.BytesType, 0),
    }

    def __init__(
        self,
        address_n: List[int] = None,
        data: bytes = None,
        global_commitment: bytes = None,
        global_pubkey: bytes = None
    ) -> None:
        self.address_n = address_n if address_n is not None else []
        self.data = data
        self.global_commitment = global_commitment
        self.global_pubkey = global_pubkey



# Automatically generated by pb2py
from .. import protobuf as p


class TxSize(p.MessageType):
    MESSAGE_WIRE_TYPE = 44
    FIELDS = {
        1: ('tx_size', p.UVarintType, 0),
    }

    def __init__(
        self,
        tx_size: int = None
    ) -> None:
        self.tx_size = tx_size



# Automatically generated by pb2py
from .. import protobuf as p
if __debug__:
    try:
        from typing import List
    except ImportError:
        List = None
from .NEMCosignatoryModification import NEMCosignatoryModification


class NEMAggregateModification(p.MessageType):
    FIELDS = {
        1: ('modifications', NEMCosignatoryModification, p.FLAG_REPEATED),
        2: ('relative_change', p.SVarintType, 0),
    }

    def __init__(
        self,
        modifications: List[NEMCosignatoryModification] = None,
        relative_change: int = None
    ) -> None:
        self.modifications = modifications if modifications is not None else []
        self.relative_change = relative_change
